/*  $Id$

    Part of XPCE
    Designed and implemented by Anjo Anjewierden and Jan Wielemaker
    E-mail: jan@swi.psy.uva.nl

    Copyright (C) 1998 University of Amsterdam. All rights reserved.
*/

:- module(pce_commander, []).
:- use_module(library(pce)).

:- pce_begin_class(pce_commander, view,
		   "Prolog toplevel in an XPCE window").

variable(output_fragment, fragment,	get,	"Output insertion point").

initialise(PC) :->
	send(PC, send_super, initialise, 'Prolog interactor'),
	send(PC, key_binding, 'RET', message(PC, send_input)),
	new(F, fragment(PC, 0, 0, output)),
	send(PC, slot, output_fragment, F),
	send(PC, prompt).

write_as_file(PC, _At:[int], Text:char_array) :->
	"Handle Input from Prolog"::
	get(PC, output_fragment, F),
	send(F, insert, @default, Text).

prompt(PC) :->
	format_view(PC, '?- ', []).

send_input(PC) :->
	send(PC, end_of_line),
	send(PC, newline),
	get(PC, output_fragment, F),
	get(F, end, Start),
	pce_open(PC, read, Fd),
	(   seek(Fd, Start, bof, _),
	    read_term(Fd, Query, []),
	    close(Fd)
	;   close(Fd),
	    send(PC, report, warning, 'Syntax error'),
	    fail
	),
	get(PC?text_buffer, size, End),
	send(F, start, End),
	send(F, length, 0),
	pce_open(PC, append, OutFd),
	current_output(OldOut),
	set_output(OutFd),
	(   catch(Query, Term, (print_message(error, Term), fail))
	->  format('~nYes~n', [])
	;   format('~nNo~n', [])
	),
	close(OutFd),
	set_output(OldOut),
	send(PC, prompt).

:- pce_end_class.

format_view(V, Fmt, Args) :-
	pce_open(V, append, Fd),
	format(Fd, Fmt, Args),
	close(Fd).
	

seek(Fd, Pos, bof, Old) :-
	stream_position(Fd,
			'$stream_position'(Old, _, _),
			'$stream_position'(Pos, 0, 0)).
